----------------------------------------------------
-- Created by Oscar Duignan <oscarduignan@gmail.com>
----------------------------------------------------

-- Comment out or add any elixir or flasks (must be the name of aura they apply)

ConsumablesTable = {
	["Battle"] = { 
        -- PREWOTLK
	    "Fel Strength Elixir",
	    "Elixir of Mastery",
        "Major Agility",
        "Major Strength",
        "Major Firepower",
        "Major Frost Power",
        "Major Shadow Power",
	    "Adepts Elixir",
	    "Healing Power",
	    "Onslaught Elixir",
        -- WOTLK
        "Accuracy",
        "Armor Piercing",
        "Deadly Strikes",
        "Expertise",
        "Lightning Speed",
        "Mighty Strength",
        "Mighty Agility",
        "Guru's Elixir",
        "Spellpower Elixir",
        "Wrath Elixir"
	},
	["Guardian"] = {
        -- PREWOTLK
	    "Elixir of Major Fortitude",
	    "Elixir of Draenic Wisdom",
	    "Elixir of Ironskin",
	    "Major Armor",
	    "Earthen Elixir",
	    "Empowerment",
	    "Greater Mana Regeneration",
	    -- WOTLK
	    "Mighty Troll's Blood Elixir",
        "Mighty Thoughts",
        "Mighty Defense",
	    "Mighty Mana Regeneration",
	    "Elixir of Spirit",
	    "Protection"
    },
    ["Flask"] = {
        -- PREWOTLK
    	"Chromatic Resistance",
    	"Distilled Wisdom",
    	"Flask of Fortification",
    	"Flask of Mighty Restoration",
    	"Flask of Relentless Assault",
    	"Flask of the Titans",
    	"Supreme Power",
    	"Chromatic Wonder",
    	"Flask of Pure Death",
    	"Flask of Blinding Light",
    	"Flask of Endless Rage",
    	-- WOTLK
    	"Flask of Pure Mojo",
    	"Flask of Stoneblood",
    	"Flask of the Frost Wyrm",
    	"Lesser Flask of Resistance"
    }
}

----------------------------------------------------
-- Sets up the slash handling
----------------------------------------------------
function CFC_OnLoad()
	SLASH_CFC1 = "/cfc";
	SlashCmdList["CFC"] = CFC_RaidCheck;

	if (DEFAULT_CHAT_FRAME) then
		DEFAULT_CHAT_FRAME:AddMessage('CheckForConsumables: AddOn Loaded.');
	end;
end;

----------------------------------------------------
-- Checks if raid members are buffed with consumables
----------------------------------------------------
function CFC_RaidCheck()
    local myZone = GetRealZoneText();
    local unitsDead = 0;
    local unitsOffline = 0;
    local unitsNotInZone = 0;
    local unitsWithout = {
        ["consumables"] = {},
        ["wellfed"] = {}
    };

    -- Must be in a raid
    if (GetNumRaidMembers() == 0) then
        DEFAULT_CHAT_FRAME:AddMessage('You are not in a raid.');
        return;
    end;

    -- BEGIN raid member loop
    for i = 1, GetNumRaidMembers() do
        local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(i);
        local hasConsumables, hasWellFed = CFC_UnitCheck(name);
        
        -- Check if unit is offline, not zoned in, or dead
	    if (not online) then unitsOffline = unitsOffline + 1;
	    elseif (myZone ~= zone) then unitsNotInZone = unitsNotInZone + 1;
	    elseif (isDead) then unitsDead = unitsDead + 1; end;

        -- true if unit is alive online and in zone
        local aliveOnlineInZone = ((not isDead) and (online) and (myZone == zone));

        -- No flask or elixirs? add to list
        if (not hasConsumables and aliveOnlineInZone) then
            unitsWithout["consumables"][i] = name;
        end;
        -- No food buff? add to list
        if (not hasWellFed and aliveOnlineInZone) then
            unitsWithout["wellfed"][i] = name;
        end;
    end;
    -- END raid member loop

    tmpConsumables = CFC_ImplodeArray(', ', unitsWithout["consumables"]);
    tmpWellFed = CFC_ImplodeArray(', ', unitsWithout["wellfed"]);

    -- Name and shame players found without flasks / elixirs
	if (string.len(tmpConsumables) > 0) then
		SendChatMessage('Missing Flask or Elixirs: '..tmpConsumables..'.', 'RAID');
	else
	    SendChatMessage('Raid is flasked.', 'RAID');
	end;

    -- Name and shame players found without well fed
	if (string.len(tmpWellFed) > 0) then
		SendChatMessage('Missing Well Fed: '..tmpWellFed..'.', 'RAID');
	else
	    SendChatMessage('Raid is well fed.', 'RAID');
	end;

    -- Report the number of dead raid members
	if (unitsDead == 1) then
	    SendChatMessage(unitsDead.." raid member is dead.", "RAID");
	elseif (unitsDead > 1) then
	    SendChatMessage(unitsDead.." raid members are dead.", "RAID");
	end;

     -- Report the number of offline raid members
	if (unitsOffline == 1) then
	    SendChatMessage(unitsOffline.." raid member is offline.", "RAID");
	elseif (unitsOffline > 1) then
	    SendChatMessage(unitsOffline.." raid members are offline.", "RAID");
	end;

    -- Report the number of raid members not in the zone
	if (unitsNotInZone == 1) then
	    SendChatMessage(unitsNotInZone.." raid member not in zone.", "RAID");
	elseif (unitsNotInZone > 1) then
	    SendChatMessage(unitsNotInZone.." raid members not in zone.", "RAID");
	end;

end;

----------------------------------------------------
-- Checks if unit is buffed with consumables
----------------------------------------------------
function CFC_UnitCheck(unitName)
    local hasFlask          = false;
    local hasBattle         = false;
    local hasGuardian       = false;
    local hasWellFed        = false;

    -- BEGIN buff loop
	for i = 0, 40 do
		local name = UnitBuff(unitName, i);
		if (name) then

            -- Check if the buff is Well Fed
            if (name == "Well Fed") then
                hasWellFed = true;
            end;

            -- Check if the buff is a flask, break loop if found
            for _, flaskName in pairs(ConsumablesTable["Flask"]) do
                if (name == flaskName) then
                    hasFlask = true;
                    break;
                end;
            end;

            -- Only check for elixirs if no flask is found
            if (not hasFlask) then
                -- Check if the buff is a Battle Elixir
                for _, elixirName in pairs(ConsumablesTable["Battle"]) do
                    if (name == elixirName) then
                        hasBattle = true;
                        break;
                    end;
                end;
        
                -- Check if the buff is a Guardian Elixir
                for _, elixirName in pairs(ConsumablesTable["Guardian"]) do
                    if (name == elixirName) then
                        hasGuardian = true;
                        break;
                    end;
                end;
            end;
            
            -- Stop the loop if unit is flasked or using elixirs
            if ((hasFlask or (hasBattle and hasGuardian)) and hasWellFed) then
                break;
            end;

		end;
	end;
	-- END buff loop

    return (hasFlask or (hasBattle and hasGuardian)), hasWellFed;
end;

----------------------------------------------------
-- Implode array function
----------------------------------------------------
function CFC_ImplodeArray(glue, haystack) 
	local output = '';
	for key, val in pairs(haystack) do
		output = output..val..glue;
	end;
	output = string.sub(output, 0, string.len(output)-string.len(glue));
	return output;
end;
